/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             String management
*  SCOPE :              String management
*  FILE :               stringx.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif

#ifndef __stringx_HXX
#define __stringx_HXX

/* IMPORTS *******************************************************************/

#ifndef __obj_HXX
#include "obj.hxx"
#endif

#ifndef __smartp_HXX
#include "smartp.hxx"
#endif

/*****************************************************************************/

class _StringX : public ObjectX {
  public :
  
  char data[1] ;
} ;


yoo_declare(_StringX) ;

//version rapide du strcmp
int strcmp_special(char* s1,char* s2) ;


class StringX   {

  private :

  char* data() const { return (ref == NULL) ? NULL : ref->data ; }

  public:

  Ref<_StringX> ref ;

  StringX() ;
  StringX(char *s);
  StringX(const StringX& s);
  // Destroy string
  void clear() { if (ref != NULL) delete (_StringX*)ref ; ref = NULL ; }

  ~StringX() { this->clear() ; }

  // Return a pointer to the string
  operator char* () const { return this->data() ; }

  // Assign the string from s
  StringX& operator=(const StringX& s) { this->assign(s); return *this ; }
  StringX& operator=(char *s) { this->assign(s); return *this ; }

  // Append s at the end of the string
  StringX& operator+=(char* s) ;

  // Prepare space to store string
  void alloc(int size) ;
  
  // Different assignement to avoid multiple copies
  void assign(char* s1) ;
  void assign(char* s1, char* s2) ;
  void assign(char* s1, char* s2, char* s3) ;
  void assign(char* s1, char* s2, char* s3, char* s4) ;
  void assign(char* s1, char* s2, char* s3, char* s4, char* s5) ;

  char& operator[](int i);
  
  // Return temporary substring
  StringX operator()(int index, int count);

  friend StringX operator+(const StringX& ,const char* );

  // Test equality of string
  int operator==(char *s) const
  {
    return(strcmp_special(this->data()==NULL?"":this->data(),s==NULL?"":s));
  }
} ;


// Compare strings

inline int operator!=(const StringX& s1, char *s2)
{
  return !(strcmp_special(s1==NULL?"":s1,s2==NULL?"":s2)) ;
}

inline int operator>(const StringX& s1, char* s2) 
{
  return(strcmp(s1==NULL?"":s1,s2==NULL?"":s2) > 0);
}

inline int operator<(const StringX& s1, char* s2)
{
  return (s2 > s1) ;
}

inline int operator<=(const StringX& s1, char* s2)
{
  return ! (s1 > s2) ;
}

inline int operator>=(const StringX& s1, char* s2) 
{
  return ! (s1 < s2) ;
}

inline ostream& operator<<(ostream& o, const StringX& s)
{
  o << (char*)s ;
  return o ;
}

inline istream& operator>>(istream& i, const StringX& s)
{
  i >> (char*)s ;
  return i ;
}

#endif
